!(function (name, context, definition) {
  if (typeof exports == 'object') {
    module.exports = definition(require);
  } else if (typeof define == 'function' && define.amd) {
    define(definition);
  } else if (typeof YUI == "function") {
    YUI.add(name, definition, '@VERSION@', {requires: []});
  } else {
    context[name] = definition();
  }
}).call(this, 'CheckboxUtil', this, function (require) {
  'use strict';

  // imports
  /**
   * Function checkboxUtility
   * Default Functionality: Adds an event handler for input labels that triggers the
   * input linked to the label with a 'click' event when the label has focus and receives
   * a keypress with 'space bar' keycode.
   * If tabIndex is false, the tabIndex for labels will not be set to 0 to allow them to
   * receive keyboard focus. Labels can be manually made focusable by adding 'tabindex=0'
   * to them in HTML/DOM.
   * If keycodes is false, the return and space bar keycodes are used by default. If a valid
   * keyCode is passed, that will be used as the impetus to trigger the input.
   * @param selector - HTML selector to find labels for, defaults to 'input[type=checkbox]'
   * @param labelTabindex - If true, sets the tabIndex for all labels to 0.
   * @param inputTabindex - If true, sets the tabIndex for all inputs to 0.
   * @param keycodes - Custom keycode(s) to trigger input click on.
   * @param focusElements - If true, checks input data attribute for a selector to make tabbable when the checkbox.
   * is checked.
   */

  function CheckboxUtil(selector, labelTabindex, inputTabindex, keycodes, focusElements) {
    // List of valid keycodes
    var validKeycodes = [8, 9, 13, 16, 17, 18, 19, 20, 27, 32, 33, 34, 35, 36, 37, 38, 39, 40, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 144, 145, 186, 187, 188, 189, 190, 191, 192, 219, 220, 221, 222];
    var defaultKeycodes = [32];

    //  Trigger Event Handler
    function triggerCheckBox(event) {
      if (this.triggerKeycodes.includes(event.keyCode)) {
        event.preventDefault();
        this.elementToTrigger.click();
      }
    }

    // Make-Untabbable Event Handler
    function makeUntabbable() {
      var checkbox = this.checkbox;
      var elements = this.elementsToHide;
      elements.forEach(function (elementToHide) {
        if (checkbox.checked) {
          elementToHide.tabIndex = 0;
        } else {
          elementToHide.tabIndex = -1;
        }
      })
    }


    // Input checking
    if (selector === undefined) {
      selector = 'input[type=checkbox]';
    } else if (!(selector instanceof String) && !(typeof selector === 'string')) {
      selector = 'input[type=checkbox]';
    }

    if (typeof focusElements !== "boolean") {
      focusElements = false;
    }

    if (inputTabindex === undefined) {
      inputTabindex = false;
    } else if (!(typeof inputTabindex === "boolean")) {
      inputTabindex = false;
    }
    if (labelTabindex === undefined) {
      labelTabindex = false;
    } else if (!(typeof labelTabindex === "boolean")) {
      labelTabindex = false;
    }
    if (keycodes === undefined) {
      keycodes = false;
    } else {
      if (Array.isArray(keycodes)) {
        keycodes.forEach(function (code) {
          if (!(validKeycodes.includes(code))) {
            keycodes = defaultKeycodes;
          }
        })
      } else if (typeof keycodes === "number") {
        if (Number.isInteger(keycodes)) {
          if (validKeycodes.includes(keycodes)) {
            keycodes = Array.of(keycodes);
          } else {
            keycodes = defaultKeycodes;
          }
        } else {
          keycodes = defaultKeycodes;
        }
      } else {
        keycodes = defaultKeycodes;
      }
    }

    // query for nodes based on passed selector
    var checkboxes = document.querySelectorAll(selector);
    // if there are any nodes found
    if (checkboxes.length) {
      checkboxes.forEach(function (checkbox) {
        //set tabIndex if desired
        if(inputTabindex) {
          checkbox.tabIndex = -1;
        }
        // find all labels for input
        var labels = document.querySelectorAll('label[for="' + checkbox.id + '"]');
        // if there are any corresponding labels found:
        if (labels.length) {
          // add our event handler to each, plus set tabindex if desired
          labels.forEach(function (label) {
            if (labelTabindex) {
              label.tabIndex = 0;
            }
            label.elementToTrigger = checkbox;
            label.triggerKeycodes = keycodes;
            label.addEventListener("keydown", triggerCheckBox)
          })
        }
        // if there's elements for which we want to tie the tabbable state to this input

        if (focusElements) {

          // add the event handler to the checkbox to hide/unhide them.

          var tabFocusElements = document.querySelectorAll(checkbox.dataset.tabHideSelector);
          if (tabFocusElements.length) {
            tabFocusElements.forEach(function (element) {
              element.tabIndex = -1;
            });
            checkbox.elementsToHide = tabFocusElements;
            checkbox.checkbox = checkbox;
            checkbox.addEventListener("change", makeUntabbable, {"passive": true});
          }
        }

      });

    }
  }

  // exports
  return CheckboxUtil;

});